@@ -28,16 +28,23 @@
 
 import os
 import pwem
+from pyworkflow.gui import FileTreeProvider
 
-from .constants import IMOD_HOME, ETOMO_CMD, DEFAULT_VERSION
-from distutils.spawn import find_executable
+from .constants import IMOD_HOME, ETOMO_CMD, DEFAULT_VERSION, VERSIONS
+from shutil import which
 from pyworkflow.gui.project.utils import OS
 
-__version__ = '3.0.8'
+__version__ = '3.0.11'
 _logo = ""
 _references = ['Kremer1996', 'Mastronarde2017']
 
 
+def getImodEnv():
+    """ This function allows to call imod outside of this plugin. """
+
+    return Plugin.getHome("IMOD-linux.sh && ")
+
+
 class Plugin(pwem.Plugin):
     _homeVar = IMOD_HOME
     _validationMsg = None
@@ -52,7 +59,7 @@ class Plugin(pwem.Plugin):
 
     @classmethod
     def _getIMODFolder(cls, version, *paths):
-        return  os.path.join(cls._getEMFolder(version, "IMOD"), *paths)
+        return os.path.join(cls._getEMFolder(version, "IMOD"), *paths)
 
     @classmethod
     def _getProgram(cls, program):
@@ -69,7 +76,7 @@ class Plugin(pwem.Plugin):
 
     @classmethod
     def getEnviron(cls):
-        env=pwem.pwutils.Environ(os.environ)
+        env = pwem.pwutils.Environ(os.environ)
         if 'IMOD_DIR' in env:
             del env['IMOD_DIR']
         if 'IMOD_PATH' in env:
@@ -83,11 +90,10 @@ class Plugin(pwem.Plugin):
         """
 
         if not cls._validationMsg:
-
             etomo = cls._getProgram(ETOMO_CMD)
 
             cls._validationMsg = [
-                "imod's %s command not found in path, please install it." % etomo] if not find_executable(
+                "imod's %s command not found in path, please install it." % etomo] if not which(
                 ETOMO_CMD) and not os.path.exists(etomo) else []
 
         return cls._validationMsg
@@ -100,39 +106,47 @@ class Plugin(pwem.Plugin):
 
     @classmethod
     def defineBinaries(cls, env):
-        IMOD_INSTALLED = 'imod_%s_installed' % DEFAULT_VERSION
 
+        for version in VERSIONS:
+            cls.installImod(env,version, version==DEFAULT_VERSION)
+
+    @classmethod
+    def installImod(cls, env, version, default):
+        IMOD_INSTALLED = 'imod_%s_installed' % version
         if 'linux' in OS.getPlatform().lower():
+            # Add jpg lib, once
+            JPEG_NAME = 'jpeg'
 
-            # Add jpg lib
-            jpeg = env.addLibrary(
-                'jpeg',
-                tar='libjpeg-turbo-1.3.1.tgz',
-                flags=['--without-simd'],
-                default=False)
+            if not env.hasTarget(JPEG_NAME):
+                jpeg = env.addLibrary(
+                    JPEG_NAME,
+                    tar='libjpeg-turbo-1.3.1.tgz',
+                    flags=['--without-simd'],
+                    default=False)
+            else:
+                jpeg = env.getTarget(JPEG_NAME)
 
             # Download .sh
             # https://bio3d.colorado.edu/imod/AMD64-RHEL5/imod_4.11.7_RHEL7-64_CUDA10.1.sh
             installationCmd = 'wget --continue http://bio3d.colorado.edu/imod/AMD64-RHEL5/' \
-                              'imod_%s_RHEL7-64_CUDA10.1.sh --no-check-certificate && ' % DEFAULT_VERSION
+                              'imod_%s_RHEL7-64_CUDA10.1.sh --no-check-certificate && ' % version
 
             # Run .sh skipping copying startup scripts (avoid sudo permissions to write to /etc/profile.d)
-            installationCmd += 'sh imod_%s_RHEL7-64_CUDA10.1.sh -dir . -yes -skip && ' % DEFAULT_VERSION
-
+            installationCmd += 'sh imod_%s_RHEL7-64_CUDA10.1.sh -dir . -yes -skip && ' % version
 
             # Create installation finished flag file
             installationCmd += 'touch %s' % IMOD_INSTALLED
 
             env.addPackage('imod',
                            deps=[jpeg],
-                           version=DEFAULT_VERSION,
+                           version=version,
                            tar='void.tgz',
                            createBuildDir=True,
-                           buildDir=cls._getEMFolder(DEFAULT_VERSION),
+                           buildDir=cls._getEMFolder(version),
                            neededProgs=cls.getDependencies(),
                            libChecks="libjpeg62",
                            commands=[(installationCmd, IMOD_INSTALLED)],
-                           default=True)
+                           default=default)
 
     @classmethod
     def runImod(cls, protocol, program, args, cwd=None):
@@ -161,3 +175,12 @@ class Plugin(pwem.Plugin):
         cmd += program
 
         return cmd
+
+
+# register file handlers to preview info in the Filebrowser....
+# Here register happens very early. Earlier than done in pwem therefore this filehandler will be the default one.
+# We can add .mrc and .mrcs but don't want to overlap with pwem ones.
+from .file_handlers import *
+
+register = FileTreeProvider.registerFileHandler
+register(ImodHandler(), '.ali', '.st', '.rec')
