@@ -23,12 +23,17 @@
 # *  e-mail address 'scipion@cnb.csic.es'
 # *
 # **************************************************************************
-
+from imod.protocols.protocol_base import OUTPUT_TILTSERIES_NAME, OUTPUT_FIDUCIAL_NO_GAPS_NAME, OUTPUT_TOMOGRAMS_NAME, \
+    OUTPUT_TS_INTERPOLATED_NAME, OUTPUT_FIDUCIAL_GAPS_NAME, OUTPUT_COORDINATES_3D_NAME
 from pyworkflow.tests import *
-from imod.protocols import *
+from pyworkflow.utils import path
+
 from pwem.emlib.image import ImageHandler
+
 import tomo
 
+from imod.protocols import *
+
 
 class TestImodBase(BaseTest):
     @classmethod
@@ -38,7 +43,7 @@ class TestImodBase(BaseTest):
     @classmethod
     def _runImportTiltSeries(cls, filesPath, pattern, voltage, magnification, sphericalAberration, amplitudeContrast,
                              samplingRate, doseInitial, dosePerFrame, anglesFrom=0, minAngle=0.0, maxAngle=0.0,
-                             stepAngle=1.0):
+                             stepAngle=1.0, tiltAxisAngle=-12.5):
         cls.protImportTS = cls.newProtocol(tomo.protocols.ProtImportTs,
                                            filesPath=filesPath,
                                            filesPattern=pattern,
@@ -52,10 +57,50 @@ class TestImodBase(BaseTest):
                                            dosePerFrame=dosePerFrame,
                                            minAngle=minAngle,
                                            maxAngle=maxAngle,
-                                           stepAngle=stepAngle)
+                                           stepAngle=stepAngle,
+                                           tiltAxisAngle=tiltAxisAngle)
         cls.launchProtocol(cls.protImportTS)
         return cls.protImportTS
 
+    @classmethod
+    def _runImportTransformationMatrix(cls, filesPath, pattern, exclusionWords, inputSetOfTiltSeries):
+        cls.protImportTM = cls.newProtocol(ProtImodImportTransformationMatrix,
+                                           filesPath=filesPath,
+                                           filesPattern=pattern,
+                                           exclusionWords=exclusionWords,
+                                           inputSetOfTiltSeries=inputSetOfTiltSeries, )
+        cls.launchProtocol(cls.protImportTM)
+        return cls.protImportTM
+
+    @classmethod
+    def _runXRaysEraser(cls, inputSoTS, peakCriterion, diffCriterion, maximumRadius, bigDiffCriterion):
+        cls.protXRaysEraser = cls.newProtocol(ProtImodXraysEraser,
+                                              inputSetOfTiltSeries=inputSoTS,
+                                              peakCriterion=peakCriterion,
+                                              diffCriterion=diffCriterion,
+                                              maximumRadius=maximumRadius,
+                                              bigDiffCriterion=bigDiffCriterion)
+        cls.launchProtocol(cls.protXRaysEraser)
+        return cls.protXRaysEraser
+
+    @classmethod
+    def _runDoseFilter(cls, inputSoTS, initialDose, inputDoseType, fixedImageDose):
+        cls.protDoseFilter = cls.newProtocol(ProtImodDoseFilter,
+                                             inputSetOfTiltSeries=inputSoTS,
+                                             initialDose=initialDose,
+                                             inputDoseType=inputDoseType,
+                                             fixedImageDose=fixedImageDose)
+        cls.launchProtocol(cls.protDoseFilter)
+        return cls.protDoseFilter
+
+    @classmethod
+    def _runExcludeViews(cls, inputSoTS, excludeViewsFile):
+        cls.protExcludeViews = cls.newProtocol(ProtImodExcludeViews,
+                                               inputSetOfTiltSeries=inputSoTS,
+                                               excludeViewsFile=excludeViewsFile)
+        cls.launchProtocol(cls.protExcludeViews)
+        return cls.protExcludeViews
+
     @classmethod
     def _runTSNormalization(cls, inputSoTS, binning, floatDensities, modeToOutput, scaleRangeToggle, scaleRangeMax,
                             scaleRangeMin, meanSdToggle, scaleMean, scaleSd, scaleMax, scaleMin):
@@ -86,14 +131,24 @@ class TestImodBase(BaseTest):
         return cls.protXcorr
 
     @classmethod
-    def _runFiducialAlignemnt(cls, inputSoTS, twoSurfaces, fiducialDiameter, numberFiducial, rotationAngle,
-                              computeAlignment, binning):
-        cls.protFiducialAlignment = cls.newProtocol(ProtImodFiducialAlignment,
+    def _runFiducialModels(cls, inputSoTS, twoSurfaces, fiducialRadius, numberFiducial, rotationAngle,
+                           shiftsNearZeroFraction) -> ProtImodFiducialModel:
+        cls.protFiducialAlignment = cls.newProtocol(ProtImodFiducialModel,
                                                     inputSetOfTiltSeries=inputSoTS,
                                                     twoSurfaces=twoSurfaces,
-                                                    fiducialDiameter=fiducialDiameter,
+                                                    fiducialRadius=fiducialRadius,
                                                     numberFiducial=numberFiducial,
                                                     rotationAngle=rotationAngle,
+                                                    shiftsNearZeroFraction=shiftsNearZeroFraction)
+        cls.launchProtocol(cls.protFiducialAlignment)
+        return cls.protFiducialAlignment
+
+    @classmethod
+    def _runFiducialAlignemnt(cls, inputSoLM, twoSurfaces, rotationAngle, computeAlignment, binning) -> ProtImodFiducialAlignment:
+        cls.protFiducialAlignment = cls.newProtocol(ProtImodFiducialAlignment,
+                                                    inputSetOfLandmarkModels=inputSoLM,
+                                                    twoSurfaces=twoSurfaces,
+                                                    rotationAngle=rotationAngle,
                                                     computeAlignment=computeAlignment,
                                                     binning=binning)
         cls.launchProtocol(cls.protFiducialAlignment)
@@ -124,8 +179,12 @@ class TestImodBase(BaseTest):
         return cls.protTomoReconstruction
 
     @classmethod
-    def _runTomoNormalization(cls, inputSetOfTomograms, binning, floatDensities, modeToOutput, scaleRangeToggle,
-                              scaleRangeMax, scaleRangeMin, meanSdToggle, scaleMean, scaleSd, scaleMax, scaleMin):
+    def _runTomoNormalization(cls,
+          inputSetOfTomograms, binning, floatDensities,
+          modeToOutput, scaleRangeToggle,
+          scaleRangeMax, scaleRangeMin, meanSdToggle,
+          scaleMean, scaleSd, scaleMax, scaleMin) -> ProtImodTomoNormalization:
+
         cls.protTomoNormalization = cls.newProtocol(ProtImodTomoNormalization,
                                                     inputSetOfTomograms=inputSetOfTomograms,
                                                     binning=binning,
@@ -142,6 +201,37 @@ class TestImodBase(BaseTest):
         cls.launchProtocol(cls.protTomoNormalization)
         return cls.protTomoNormalization
 
+    @classmethod
+    def _runGoldBeadPiker3D(cls, inputSetOfTomograms, beadDiameter, beadsColor, minRelativeStrength, minSpacing) -> ProtImodGoldBeadPicker3d:
+        cls.protGoldBeadPicker3d = cls.newProtocol(ProtImodGoldBeadPicker3d,
+                                                   inputSetOfTomograms=inputSetOfTomograms,
+                                                   beadDiameter=beadDiameter,
+                                                   beadsColor=beadsColor,
+                                                   minRelativeStrength=minRelativeStrength,
+                                                   minSpacing=minSpacing)
+        cls.launchProtocol(cls.protGoldBeadPicker3d)
+        return cls.protGoldBeadPicker3d
+
+    @classmethod
+    def _runTomoProjection(cls, inputSetOfTomograms, minAngle, maxAngle, stepAngle, rotationAxis):
+        cls.protTomoProjection = cls.newProtocol(ProtImodTomoProjection,
+                                                 inputSetOfTomograms=inputSetOfTomograms,
+                                                 minAngle=minAngle,
+                                                 maxAngle=maxAngle,
+                                                 stepAngle=stepAngle,
+                                                 rotationAxis=rotationAxis)
+        cls.launchProtocol(cls.protTomoProjection)
+        return cls.protTomoProjection
+
+    @classmethod
+    def _runImportSetOfCtfSeries(cls, filesPath, filesPattern, inputSetOfTiltSeries):
+        cls.protImportSetOfCtfSeries = cls.newProtocol(ProtImodImportSetOfCtfTomoSeries,
+                                                       filesPath=filesPath,
+                                                       filesPattern=filesPattern,
+                                                       inputSetOfTiltSeries=inputSetOfTiltSeries)
+        cls.launchProtocol(cls.protImportSetOfCtfSeries)
+        return cls.protImportSetOfCtfSeries
+
     @classmethod
     def _runCTFEstimation(cls, inputSoTS, defocusTol, expectedDefocusOrigin, expectedDefocusValue, expectedDefocusFile,
                           axisAngle, interactiveMode, leftDefTol, rightDefTol, tileSize, angleStep, angleRange,
@@ -149,7 +239,7 @@ class TestImodBase(BaseTest):
                           findAstigPhaseCutonToggle, phaseShiftAstigmatism, cutOnFrequencyAstigmatism,
                           minimumViewsAstigmatism, minimumViewsPhaseShift, numberSectorsAstigmatism,
                           maximumAstigmatism):
-        cls.protCTFEstimation = cls.newProtocol(ProtImodCtfEstimation,
+        cls.protCTFEstimation = cls.newProtocol(ProtImodAutomaticCtfEstimation,
                                                 inputSet=inputSoTS,
                                                 defocusTol=defocusTol,
                                                 expectedDefocusOrigin=expectedDefocusOrigin,
@@ -187,17 +277,6 @@ class TestImodBase(BaseTest):
         cls.launchProtocol(cls.protCTFCorrection)
         return cls.protCTFCorrection
 
-    @classmethod
-    def _runTomoProjection(cls, inputSetOfTomograms, minAngle, maxAngle, stepAngle, rotationAxis):
-        cls.protTomoProjection = cls.newProtocol(ProtImodTomoProjection,
-                                                 inputSetOfTomograms=inputSetOfTomograms,
-                                                 minAngle=minAngle,
-                                                 maxAngle=maxAngle,
-                                                 stepAngle=stepAngle,
-                                                 rotationAxis=rotationAxis)
-        cls.launchProtocol(cls.protTomoProjection)
-        return cls.protTomoProjection
-
 
 class TestImodReconstructionWorkflow(TestImodBase):
     @classmethod
@@ -207,6 +286,12 @@ class TestImodReconstructionWorkflow(TestImodBase):
         cls.inputDataSet = DataSet.getDataSet('tomo-em')
         cls.inputSoTS = cls.inputDataSet.getFile('ts1')
 
+        cls.excludeViewsFile = cls.inputDataSet.getFile('excludeViewsFile')
+
+        cls.inputTMFolder = os.path.split(cls.inputDataSet.getFile('tm1'))[0]
+
+        cls.excludeViewsOutputSizes = {'a': 57, 'b': 56}
+
         cls.binningTsNormalization = 2
 
         cls.binningPrealignment = 2
@@ -233,7 +318,26 @@ class TestImodReconstructionWorkflow(TestImodBase):
                                                     maxAngle=65.0,
                                                     stepAngle=2.0)
 
-        cls.protTSNormalization = cls._runTSNormalization(inputSoTS=cls.protImportTS.outputTiltSeries,
+        cls.protImportTM = cls._runImportTransformationMatrix(filesPath=cls.inputTMFolder,
+                                                              pattern="BB*.prexg",
+                                                              exclusionWords='',
+                                                              inputSetOfTiltSeries=cls.protImportTS.outputTiltSeries)
+
+        cls.protXRaysEraser = cls._runXRaysEraser(inputSoTS=cls.protImportTS.outputTiltSeries,
+                                                  peakCriterion=8.0,
+                                                  diffCriterion=6.0,
+                                                  maximumRadius=4.2,
+                                                  bigDiffCriterion=19)
+
+        cls.protDoseFilter = cls._runDoseFilter(inputSoTS=cls.protXRaysEraser.TiltSeries,
+                                                initialDose=0,
+                                                inputDoseType=1,
+                                                fixedImageDose=0.2)
+
+        cls.protExcludeViews = cls._runExcludeViews(inputSoTS=cls.protXRaysEraser.TiltSeries,
+                                                    excludeViewsFile=cls.excludeViewsFile)
+
+        cls.protTSNormalization = cls._runTSNormalization(inputSoTS=cls.protDoseFilter.TiltSeries,
                                                           binning=cls.binningTsNormalization,
                                                           floatDensities=0,
                                                           modeToOutput=0,
@@ -246,25 +350,30 @@ class TestImodReconstructionWorkflow(TestImodBase):
                                                           scaleMax=255,
                                                           scaleMin=0)
 
-        cls.protXcorr = cls._runXcorrPrealignment(inputSoTS=cls.protImportTS.outputTiltSeries,
+        cls.protXcorr = cls._runXcorrPrealignment(inputSoTS=cls.protDoseFilter.TiltSeries,
                                                   computeAlignmentToggle=0,
                                                   binning=cls.binningPrealignment,
                                                   rotationAngle=-12.5)
 
-        cls.protFiducialAlignment = cls._runFiducialAlignemnt(inputSoTS=cls.protXcorr.outputSetOfTiltSeries,
+        cls.protFiducialModels = cls._runFiducialModels(inputSoTS=cls.protXcorr.TiltSeries,
+                                                        twoSurfaces=0,
+                                                        fiducialRadius=4.95,
+                                                        numberFiducial=25,
+                                                        rotationAngle=-12.5,
+                                                        shiftsNearZeroFraction=0.2)
+
+        cls.protFiducialAlignment = cls._runFiducialAlignemnt(inputSoLM=cls.protFiducialModels.FiducialModelGaps,
                                                               twoSurfaces=0,
-                                                              fiducialDiameter=4.95,
-                                                              numberFiducial=25,
-                                                              rotationAngle=0,
+                                                              rotationAngle=-12.5,
                                                               computeAlignment=0,
                                                               binning=cls.binningFiducialAlignment)
 
         cls.protApplyTransformationMatrix = \
-            cls._runApplyTransformationMatrix(inputSoTS=cls.protFiducialAlignment.outputSetOfTiltSeries,
+            cls._runApplyTransformationMatrix(inputSoTS=cls.protFiducialAlignment.TiltSeries,
                                               binning=cls.binningApplyTransformMatrix)
 
         cls.protTomoReconstruction = \
-            cls._runTomoReconstruction(inputSoTS=cls.protFiducialAlignment.outputSetOfTiltSeries,
+            cls._runTomoReconstruction(inputSoTS=cls.protFiducialAlignment.TiltSeries,
                                        tomoThickness=cls.thicknessTomo,
                                        tomoShiftX=0.0,
                                        tomoShiftZ=0.0,
@@ -275,7 +384,7 @@ class TestImodReconstructionWorkflow(TestImodBase):
                                        radialSecondParameter=0.035)
 
         cls.protTomoNormalization = \
-            cls._runTomoNormalization(inputSetOfTomograms=cls.protTomoReconstruction.outputSetOfTomograms,
+            cls._runTomoNormalization(inputSetOfTomograms=cls.protTomoReconstruction.Tomograms,
                                       binning=cls.binningTomoNormalization,
                                       floatDensities=0,
                                       modeToOutput=0,
@@ -288,147 +397,209 @@ class TestImodReconstructionWorkflow(TestImodBase):
                                       scaleMax=255,
                                       scaleMin=0)
 
+        cls.protGoldBeadPicker3D = \
+            cls._runGoldBeadPiker3D(inputSetOfTomograms=cls.protTomoReconstruction.Tomograms,
+                                    beadDiameter=10,
+                                    beadsColor=0,
+                                    minRelativeStrength=0.05,
+                                    minSpacing=0.9)
+
         cls.protTomoProjection = \
-            cls._runTomoProjection(inputSetOfTomograms=cls.protTomoNormalization.outputNormalizedSetOfTomograms,
+            cls._runTomoProjection(inputSetOfTomograms=cls.protTomoNormalization.Tomograms,
                                    minAngle=-60.0,
                                    maxAngle=60.0,
                                    stepAngle=2.0,
                                    rotationAxis=1)
 
-    def test_normalizationOutputTS(self):
-        self.assertIsNotNone(self.protTSNormalization.outputNormalizedSetOfTiltSeries)
+    def test_importTMOutput(self):
 
-        tsId = self.protTSNormalization.outputNormalizedSetOfTiltSeries.getFirstItem().getTsId()
+        tseries = self.protImportTM.TiltSeries
+        self.assertSetSize(tseries, size=2)
+
+        for ts in tseries:
+            self.assertTrue(ts.getFirstItem().hasTransform())
+
+    def test_doseFilterOutputTS(self):
+        ts = self.protDoseFilter.TiltSeries
+        self.assertSetSize(ts)
+
+        tsId = ts.getFirstItem().getTsId()
+
+        self.assertTrue(os.path.exists(os.path.join(self.protDoseFilter._getExtraPath(tsId), "BB" + tsId + ".st")))
+
+    def test_xRaysEraserOutputTS(self):
+
+        ts = self.protXRaysEraser.TiltSeries
+        self.assertSetSize(ts)
+
+        tsId = ts.getFirstItem().getTsId()
+
+        self.assertTrue(os.path.exists(os.path.join(self.protXRaysEraser._getExtraPath(tsId), "BB" + tsId + ".st")))
+
+    def test_excludeViewsOutputTS(self):
+
+        ts = self.protExcludeViews.TiltSeries
+        self.assertSetSize(ts)
+
+        tsId = ts.getFirstItem().getTsId()
+
+        self.assertTrue(os.path.exists(os.path.join(self.protExcludeViews._getExtraPath(tsId), "BB" + tsId + ".st")))
+
+        for index, tsOut in enumerate(ts):
+            self.assertEqual(tsOut.getSize(), self.excludeViewsOutputSizes[tsOut.getTsId()])
+
+    def test_normalizationOutputTS(self):
+
+        ts = self.protTSNormalization.TiltSeries
+        self.assertSetSize(ts)
+
+        tsId = ts.getFirstItem().getTsId()
 
         self.assertTrue(os.path.exists(os.path.join(self.protTSNormalization._getExtraPath(tsId), "BB" + tsId + ".st")))
 
-    def test_normalizationOutputTSSamplingRate(self):
         inSamplingRate = self.protTSNormalization.inputSetOfTiltSeries.get().getSamplingRate()
-        outSamplingRate = self.protTSNormalization.outputNormalizedSetOfTiltSeries.getSamplingRate()
+        outSamplingRate = ts.getSamplingRate()
 
-        self.assertTrue(inSamplingRate * self.binningTsNormalization == outSamplingRate)
+        self.assertEqual(inSamplingRate * self.binningTsNormalization, outSamplingRate)
 
     def test_prealignmentOutputTS(self):
-        self.assertIsNotNone(self.protXcorr.outputSetOfTiltSeries)
 
-        tsId = self.protXcorr.outputSetOfTiltSeries.getFirstItem().getTsId()
+        ts = self.protXcorr.TiltSeries
+        self.assertSetSize(ts)
+
+        tsId = ts.getFirstItem().getTsId()
         outputLocation = os.path.join(self.protXcorr._getExtraPath(tsId), "BB" + tsId + ".st")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_prealignmentTransformMatrixOutputTS(self):
-        self.assertIsNotNone(self.protXcorr.outputSetOfTiltSeries.getFirstItem().getFirstItem().getTransform())
+        self.assertIsNotNone(ts.getFirstItem().getFirstItem().getTransform())
 
     def test_prealignmentOutputInterpolatedTS(self):
-        self.assertIsNotNone(self.protXcorr.outputInterpolatedSetOfTiltSeries)
 
-        tsId = self.protXcorr.outputInterpolatedSetOfTiltSeries.getFirstItem().getTsId()
+        ts = self.protXcorr.InterpolatedTiltSeries
+        self.assertSetSize(ts)
+
+        tsId = ts.getFirstItem().getTsId()
         outputLocation = os.path.join(self.protXcorr._getExtraPath(tsId), "BB" + tsId + ".st")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_prealignmentOutputInterpolatedTSSamplingRate(self):
         inSamplingRate = self.protXcorr.inputSetOfTiltSeries.get().getSamplingRate()
-        outSamplingRate = self.protXcorr.outputInterpolatedSetOfTiltSeries.getSamplingRate()
+        outSamplingRate = ts.getSamplingRate()
+
+        self.assertEqual(inSamplingRate * self.binningPrealignment, outSamplingRate)
+
+    def test_fiducialModelstOutputFiducialModelGaps(self):
+
+        output = self.protFiducialModels.FiducialModelGaps
+        self.assertSetSize(output, size=2)
+
+        tsId = output.getFirstItem().getTsId()
+        outputLocationImod = os.path.join(self.protFiducialModels._getExtraPath(tsId), "BB" + tsId + "_gaps.fid")
+        outputLocationScipion = os.path.join(self.protFiducialModels._getExtraPath(tsId), "BB" + tsId + "_gaps.sfid")
+
+        self.assertTrue(os.path.exists(outputLocationImod))
+        self.assertTrue(os.path.exists(outputLocationScipion))
 
-        self.assertTrue(inSamplingRate * self.binningPrealignment == outSamplingRate)
 
     def test_fiducialAlignmentOutputTS(self):
-        self.assertIsNotNone(self.protFiducialAlignment.outputSetOfTiltSeries)
 
-        tsId = self.protFiducialAlignment.outputSetOfTiltSeries.getFirstItem().getTsId()
+        output = self.protFiducialAlignment.TiltSeries
+        self.assertSetSize(output, size=2)
+
+        tsId = output.getFirstItem().getTsId()
         outputLocation = os.path.join(self.protFiducialAlignment._getExtraPath(tsId), "BB" + tsId + ".st")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_fiducialAlignmentTransformMatrixOutputTS(self):
-        self.assertIsNotNone(
-            self.protFiducialAlignment.outputSetOfTiltSeries.getFirstItem().getFirstItem().getTransform())
+        self.assertIsNotNone(output.getFirstItem().getFirstItem().getTransform())
 
-    def test_fiducialAlignmentOutputInterpolatedTS(self):
-        self.assertIsNotNone(self.protFiducialAlignment.outputInterpolatedSetOfTiltSeries)
+        # Interpolated
+        output = self.protFiducialAlignment.InterpolatedTiltSeries
+        self.assertSetSize(output, size=2)
 
-        tsId = self.protFiducialAlignment.outputInterpolatedSetOfTiltSeries.getFirstItem().getTsId()
+        tsId = output.getFirstItem().getTsId()
         outputLocation = os.path.join(self.protFiducialAlignment._getExtraPath(tsId), "BB" + tsId + ".st")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_fiducialAlignmentOutputInterpolatedTSSamplingRate(self):
-        inSamplingRate = self.protFiducialAlignment.inputSetOfTiltSeries.get().getSamplingRate()
-        outSamplingRate = self.protFiducialAlignment.outputInterpolatedSetOfTiltSeries.getSamplingRate()
+        inputSoTS = self.protFiducialAlignment.inputSetOfLandmarkModels.get().getSetOfTiltSeries()
 
-        self.assertTrue(inSamplingRate * self.binningFiducialAlignment == outSamplingRate)
+        inSamplingRate = inputSoTS.getSamplingRate()
+        outSamplingRate = output.getSamplingRate()
 
-    # def test_fiducialAlignmentOutputFiducialModelGaps(self):
-    #     self.assertIsNotNone(self.protFiducialAlignment.outputFiducialModelGaps)
-    #
-    #     tsId = self.protFiducialAlignment.outputInterpolatedSetOfTiltSeries.getFirstItem().getTsId()
-    #     outputLocation = os.path.join(self.protFiducialAlignment._getExtraPath(tsId), "BB" + tsId + "_resid.txt")
-    #
-    #     self.assertTrue(os.path.exists(outputLocation))
-    #
-    # def test_fiducialAlignmentOutputFiducialModelGapsSize(self):
-    #     self.assertTrue(self.protFiducialAlignment.outputFiducialModelGaps.getSize() == 2)
+        self.assertEqual(inSamplingRate * self.binningFiducialAlignment, outSamplingRate)
 
-    def test_fiducialAlignmentOutputFiducialModelNoGaps(self):
-        self.assertIsNotNone(self.protFiducialAlignment.outputFiducialModelNoGaps)
 
-        tsId = self.protFiducialAlignment.outputInterpolatedSetOfTiltSeries.getFirstItem().getTsId()
+        output = self.protFiducialAlignment.FiducialModelNoGaps
+        self.assertSetSize(output, size=2)
+
+        tsId = output.getFirstItem().getTsId()
         outputLocation = os.path.join(self.protFiducialAlignment._getExtraPath(tsId), "BB" + tsId + "_noGaps.sfid")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_fiducialAlignmentOutputFiducialModelNoGapsSize(self):
-        self.assertTrue(self.protFiducialAlignment.outputFiducialModelNoGaps.getSize() == 2)
+        # Output coordinates
+        output = self.protFiducialAlignment.TiltSeriesCoordinates
+        self.assertSetSize(output)
 
-    def test_fiducialAlignmentOutputCoordinates3D(self):
-        self.assertIsNotNone(self.protFiducialAlignment.outputSetOfCoordinates3D)
-
-        tsId = self.protFiducialAlignment.outputInterpolatedSetOfTiltSeries.getFirstItem().getTsId()
+        tsId = output.getFirstItem().getTsId()
         outputLocation = os.path.join(self.protFiducialAlignment._getExtraPath(tsId), "BB" + tsId + "_fid.xyz")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_fiducialAlignmentOutputCoordinates3DSize(self):
-        tolerance = 2
-        expectedSize = 50
+        tolerance = 20
+        expectedSize = 35
 
-        self.assertTrue(
-            abs(self.protFiducialAlignment.outputSetOfCoordinates3D.getSize() - expectedSize) <= tolerance)
+        self.assertTrue(abs(output.getSize() - expectedSize) <= tolerance)
 
     def test_applyTransformationMatrixOutputInterpolatedTS(self):
-        self.assertIsNotNone(self.protApplyTransformationMatrix.outputInterpolatedSetOfTiltSeries)
 
-        tsId = self.protFiducialAlignment.outputInterpolatedSetOfTiltSeries.getFirstItem().getTsId()
+        output = self.protApplyTransformationMatrix.InterpolatedTiltSeries
+        self.assertSetSize(output, size=2)
+
+        tsId = output.getFirstItem().getTsId()
         outputLocation = os.path.join(self.protApplyTransformationMatrix._getExtraPath(tsId), "BB" + tsId + ".st")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_applyTransformationMatrixOutputInterpolatedTSSamplingRate(self):
         inSamplingRate = self.protApplyTransformationMatrix.inputSetOfTiltSeries.get().getSamplingRate()
-        outSamplingRate = self.protApplyTransformationMatrix.outputInterpolatedSetOfTiltSeries.getSamplingRate()
+        outSamplingRate = output.getSamplingRate()
 
-        self.assertTrue(inSamplingRate * self.binningApplyTransformMatrix == outSamplingRate)
+        self.assertEqual(inSamplingRate * self.binningApplyTransformMatrix, outSamplingRate)
 
     def test_tomoReconstructionOutputTomogram(self):
-        self.assertIsNotNone(self.protTomoReconstruction.outputSetOfTomograms)
+
+        output = self.protTomoReconstruction.Tomograms
+        self.assertSetSize(output, size=2)
 
         tomoId = self.protTomoReconstruction.inputSetOfTiltSeries.get().getFirstItem().getTsId()
         outputLocation = os.path.join(self.protTomoReconstruction._getExtraPath(tomoId), "BB" + tomoId + ".mrc")
 
         self.assertTrue(os.path.exists(outputLocation))
 
-    def test_tomoReconstructionOutputTomogramSize(self):
-        self.assertEqual(self.protTomoReconstruction.outputSetOfTomograms.getSize(), 2)
-
-    def test_tomoReconstructionOutputTomogramDimensions(self):
+        # Dimensions
         ih = ImageHandler()
-        outputDimensions = ih.getDimensions(self.protTomoReconstruction.outputSetOfTomograms.getFirstItem())
+        outputDimensions = ih.getDimensions(output.getFirstItem())
+
+        self.assertEqual(outputDimensions, (512, 512, self.thicknessTomo, 1))
+
+    def test_goldBeadPeaker3DOutput(self):
+
+        output = self.protGoldBeadPicker3D.Coordinates3D
+        self.assertSetSize(output, size=52 , diffDelta=30)
+
+
+        tomoId = self.protGoldBeadPicker3D.inputSetOfTomograms.get().getFirstItem().getTsId()
+        location = self.protGoldBeadPicker3D._getExtraPath("BB" + tomoId)
+
+        self.assertTrue(os.path.exists(os.path.join(location, "BB" + tomoId + ".mod")))
 
-        self.assertTrue(outputDimensions == (512, 512, self.thicknessTomo, 1))
 
     def test_tomoNormalizationOutput(self):
-        self.assertIsNotNone(self.protTomoNormalization.outputNormalizedSetOfTomograms)
+
+        output = self.protTomoNormalization.Tomograms
+        self.assertSetSize(output, size=2)
 
         location = self.protTomoNormalization.inputSetOfTomograms.get().getFirstItem().getLocation()[1]
         fileName, _ = os.path.splitext(location)
@@ -436,30 +607,27 @@ class TestImodReconstructionWorkflow(TestImodBase):
 
         self.assertTrue(os.path.exists(os.path.join(self.protTomoNormalization._getExtraPath(tomoId), tomoId + ".mrc")))
 
-    def test_tomoNormalizationOutputSize(self):
-        self.assertTrue(self.protTomoNormalization.outputNormalizedSetOfTomograms.getSize() == 2)
-
-    def test_tomoNormalizationOutputSamplingRate(self):
         inSamplingRate = self.protTomoNormalization.inputSetOfTomograms.get().getSamplingRate()
-        outSamplingRate = self.protTomoNormalization.outputNormalizedSetOfTomograms.getSamplingRate()
+        outSamplingRate = output.getSamplingRate()
 
-        self.assertTrue(inSamplingRate * self.binningTomoNormalization == outSamplingRate)
+        self.assertEqual(inSamplingRate * self.binningTomoNormalization, outSamplingRate)
 
     def test_tomoProjectionOutputTiltSeriesSize(self):
-        self.assertTrue(self.protTomoProjection.outputProjectedSetOfTiltSeries.getSize() == 2)
 
-    def test_tomoProjectionOutputTiltSeriesDimension(self):
+        output = self.protTomoProjection.TiltSeries
+        self.assertSetSize(output, size=2)
+
+        # Dimensions
         ih = ImageHandler()
-        outputDimensions = ih.getDimensions(
-            self.protTomoProjection.outputProjectedSetOfTiltSeries.getFirstItem().getFirstItem().getFileName())
+        outputDimensions = ih.getDimensions(output.getFirstItem().getFirstItem().getFileName())
 
-        self.assertTrue(outputDimensions == (256, 256, 61, 1))
+        self.assertEqual(outputDimensions, (256, 256, 61, 1))
 
-    def test_tomoProjectionOutputTiltSeriesSamplingRate(self):
+        # Sampling rate
         inSamplingRate = self.protTomoProjection.inputSetOfTomograms.get().getSamplingRate()
-        outSamplingRate = self.protTomoProjection.outputProjectedSetOfTiltSeries.getSamplingRate()
+        outSamplingRate = output.getSamplingRate()
 
-        self.assertTrue(inSamplingRate == outSamplingRate)
+        self.assertEqual(inSamplingRate, outSamplingRate)
 
 
 class TestImodCTFCorrectionWorkflow(TestImodBase):
@@ -470,8 +638,17 @@ class TestImodCTFCorrectionWorkflow(TestImodBase):
         cls.inputDataSet = DataSet.getDataSet('tutorialDataImodCTF')
         cls.inputSoTS = cls.inputDataSet.getFile('tsCtf1')
 
+        cls.inputCtfFile = cls.inputDataSet.getFile('inputCtfFile')
+
+        # Create links to the input tilt-series and its associated mdoc file to test the protocols with a set of two
+        # elements to make the tests more robust
+        linkTs = os.path.join(os.path.split(cls.inputSoTS)[0], "WTI042413_1series4_copy.st")
+
+        if not os.path.exists(linkTs):
+            path.createLink(cls.inputSoTS, linkTs)
+
         cls.protImportTS = cls._runImportTiltSeries(filesPath=os.path.split(cls.inputSoTS)[0],
-                                                    pattern="WTI042413_1series4.st",
+                                                    pattern="*.mdoc",
                                                     anglesFrom=0,
                                                     voltage=300,
                                                     magnification=50000,
@@ -481,6 +658,11 @@ class TestImodCTFCorrectionWorkflow(TestImodBase):
                                                     doseInitial=0,
                                                     dosePerFrame=0.3)
 
+        cls.protImportSetOfCtfSeries = \
+            cls._runImportSetOfCtfSeries(filesPath=os.path.split(cls.inputCtfFile)[0],
+                                         filesPattern='WTI042413_1series4.defocus',
+                                         inputSetOfTiltSeries=cls.protImportTS.outputTiltSeries)
+
         cls.protCTFEstimation = cls._runCTFEstimation(inputSoTS=cls.protImportTS.outputTiltSeries,
                                                       defocusTol=200.0,
                                                       expectedDefocusOrigin=0,
@@ -507,25 +689,33 @@ class TestImodCTFCorrectionWorkflow(TestImodBase):
                                                       maximumAstigmatism=1.2)
 
         cls.protCTFCorrection = cls._runCTFCorrection(inputSetOfTiltSeries=cls.protImportTS.outputTiltSeries,
-                                                      inputSetOfCtfTomoSeries=cls.protCTFEstimation.outputSetOfCTFTomoSeries,
+                                                      inputSetOfCtfTomoSeries=cls.protCTFEstimation.CTFTomoSeries,
                                                       defocusTol=200,
                                                       interpolationWidth=15)
 
+    def test_importCtfTomoSeriesOutput(self):
+        self.assertSetSize(self.protImportSetOfCtfSeries.CTFTomoSeries, size=1)
+
+
     def test_ctfEstimationOutputSize(self):
-        self.assertIsNotNone(self.protCTFEstimation.outputSetOfCTFTomoSeries)
+        self.assertSetSize(self.protCTFEstimation.CTFTomoSeries, size=2)
 
-        self.assertTrue(self.protCTFEstimation.outputSetOfCTFTomoSeries.getSize() == 1)
 
     def test_ctfEstimationOutputDefocusFile(self):
-        tsId = self.protCTFEstimation.inputSet.get().getFirstItem().getTsId()
-        defocusFile = os.path.join(self.protCTFEstimation._getExtraPath(tsId), '%s.defocus' % tsId)
+        for ts in self.protCTFEstimation.inputSet.get():
+            tsId = ts.getTsId()
+            defocusFile = os.path.join(self.protCTFEstimation._getExtraPath(tsId), '%s.defocus' % tsId)
 
-        self.assertTrue(os.path.exists(defocusFile))
+            self.assertTrue(os.path.exists(defocusFile))
 
     def test_ctfCorrectionOutput(self):
-        self.assertIsNotNone(self.protCTFCorrection.outputCtfCorrectedSetOfTiltSeries)
 
-        tsId = self.protCTFCorrection.outputCtfCorrectedSetOfTiltSeries.getFirstItem().getTsId()
-        outputLocation = os.path.join(self.protCTFCorrection._getExtraPath(tsId), "WTI042413_1series4.st")
+        output = self.protCTFCorrection.TiltSeries
+        self.assertSetSize(output, size=2)
 
-        self.assertTrue(os.path.exists(outputLocation))
+
+        for ts in output:
+            tsId = ts.getTsId()
+            outputLocation = os.path.join(self.protCTFCorrection._getExtraPath(tsId), '%s.st' % tsId)
+
+            self.assertTrue(os.path.exists(outputLocation))
